/* A Bison parser, made by GNU Bison 3.3.2.  */

/* Bison implementation for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Undocumented macros, especially those whose name start with YY_,
   are private implementation details.  Do not rely on them.  */

/* Identify Bison output.  */
#define YYBISON 1

/* Bison version.  */
#define YYBISON_VERSION "3.3.2"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1




/* First part of user prologue.  */
#line 1 "configfile.y" /* yacc.c:337  */

/*
**  A yacc input file for the innfeed config file.
**
**  Written by James Brister <brister@vix.com>
**
**  This file contains the heart of the innfeed configuration parser, written
**  in yacc.  It uses an external lexer generated by flex.
**
**  clang-format does not work off-the-shelf with mixed C and yacc code;
**  if it is run, the "%{" and "%}" lines should be commented so as not
**  to add extra indentation.
*/

#include "portable/system.h"

#include "innfeed.h"
#include <ctype.h>
#include <errno.h>
#include <syslog.h>

#include "inn/libinn.h"
#include "inn/messages.h"

#include "configfile.h"
#include "misc.h"

/* flex does not declare yydebug as static, and uses superfluous breaks. */
#if defined(__llvm__) || defined(__clang__)
# pragma GCC diagnostic ignored "-Wmissing-variable-declarations"
# pragma GCC diagnostic ignored "-Wunreachable-code-break"
#endif

#define UNKNOWN_SCOPE_TYPE "line %d: unknown scope type: %s"
#define SYNTAX_ERROR       "line %d: syntax error"

scope *topScope = NULL;
static scope *currScope = NULL;
char *errbuff = NULL;

static void appendName(scope *s, char *p, size_t len);
static char *valueScopedName(value *v);
static void freeValue(value *v);
static char *checkName(scope *s, const char *name);
static void addValue(scope *s, value *v);
static char *addScope(scope *s, const char *name, scope *val);
static void printScope(FILE *fp, scope *s, int indent);
static void printValue(FILE *fp, value *v, int indent);
static scope *newScope(const char *type);

/* Interface between lexer and parser. */
int yylex(void);

int yyerror(const char *s);
int yywrap(void);

int
getBool(scope *s, const char *name, int *rval, int inherit)
{
    value *v = findValue(s, name, inherit);

    if (v == NULL)
        return 0;
    else if (v->type != boolval)
        return 0;

    *rval = v->v.bool_val;
    return 1;
}


int
getString(scope *s, const char *name, char **rval, int inherit)
{
    value *v = findValue(s, name, inherit);

    if (v == NULL)
        return 0;
    else if (v->type != stringval)
        return 0;

    *rval = xstrdup(v->v.charp_val);
    return 1;
}


int
getReal(scope *s, const char *name, double *rval, int inherit)
{
    value *v = findValue(s, name, inherit);

    if (v == NULL)
        return 0;
    else if (v->type != realval)
        return 0;

    *rval = v->v.real_val;
    return 1;
}

int
getInteger(scope *s, const char *name, long *rval, int inherit)
{
    value *v = findValue(s, name, inherit);

    if (v == NULL)
        return 0;
    else if (v->type != intval)
        return 0;

    *rval = v->v.int_val;
    return 1;
}

void
freeScopeTree(scope *s)
{
    int i;

    if (s == NULL)
        return;

    if (s->parent == NULL && s->me != NULL) { /* top level scope */
        free(s->me->name);
        free(s->me);
    }


    for (i = 0; i < s->value_idx; i++)
        if (s->values[i] != NULL)
            freeValue(s->values[i]);

    free(s->values);
    free(s->scope_type);

    s->parent = NULL;
    s->values = NULL;

    free(s);
}


char *
addInteger(scope *s, const char *name, long val)
{
    value *v;
    char *error;

    if ((error = checkName(currScope, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = intval;
    v->v.int_val = val;

    addValue(s, v);

    return NULL;
}

char *
addChar(scope *s, const char *name, char val)
{
    value *v;
    char *error;

    if ((error = checkName(currScope, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = charval;
    v->v.char_val = val;

    addValue(s, v);

    return NULL;
}

char *
addBoolean(scope *s, const char *name, int val)
{
    value *v;
    char *error;

    if ((error = checkName(currScope, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = boolval;
    v->v.bool_val = val;

    addValue(s, v);

    return NULL;
}

char *
addReal(scope *s, const char *name, double val)
{
    value *v;
    char *error;

    if ((error = checkName(currScope, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = realval;
    v->v.real_val = val;

    addValue(s, v);

    return NULL;
}

char *
addString(scope *s, const char *name, const char *val)
{
    value *v;
    char *error;

    if ((error = checkName(currScope, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = stringval;
    v->v.charp_val = xstrdup(val);

    addValue(s, v);

    return NULL;
}

value *
findValue(scope *s, const char *name, int inherit)
{
    const char *p;

    if (name == NULL || *name == '\0')
        return NULL;

    if (*name == ':')
        return findValue(topScope, name + 1, 0);
    else if (s == NULL)
        return findValue(topScope, name, 0);
    else {
        int i;

        if ((p = strchr(name, ':')) == NULL)
            p = name + strlen(name);

        for (i = 0; i < s->value_idx; i++) {
            if (strlen(s->values[i]->name) == (size_t)(p - name)
                && strncmp(s->values[i]->name, name, p - name) == 0) {
                if (*p == '\0') /* last segment of name */
                    return s->values[i];
                else if (s->values[i]->type != scopeval)
                    errbuff = xstrdup("Component not a scope");
                else
                    return findValue(s->values[i]->v.scope_val, p + 1, 0);
            }
        }

        /* not in this scope. Go up if inheriting values and only if no ':'
           in name */
        if (inherit && *p == '\0')
            return findValue(s->parent, name, inherit);
    }

    return NULL;
}

/* find the scope that name belongs to. If mustExist is true then the name
   must be a fully scoped name of a value. relative scopes start at s. */
scope *
findScope(scope *s, const char *name, int mustExist)
{
    scope *p = NULL;
    char *q;
    int i;


    if ((q = strchr(name, ':')) == NULL) {
        if (!mustExist)
            p = s;
        else
            for (i = 0; p == NULL && i < s->value_idx; i++)
                if (strcmp(s->values[i]->name, name) == 0)
                    p = s;

        return p;
    } else if (*name == ':') {
        while (s->parent != NULL)
            s = s->parent;

        return findScope(s, name + 1, mustExist);
    } else {
        for (i = 0; i < s->value_idx; i++)
            if (strncmp(s->values[i]->name, name, q - name) == 0)
                if (s->values[i]->type == scopeval)
                    return findScope(s->values[i]->v.scope_val, q + 1,
                                     mustExist);
    }

    return NULL;
}

/****************************************************************************/
/*                                                                          */
/****************************************************************************/


static void
appendName(scope *s, char *p, size_t len)
{
    if (s == NULL)
        return;
    else {
        appendName(s->parent, p, len);
        strlcat(p, s->me->name, len);
        strlcat(p, ":", len);
    }
}

static char *
valueScopedName(value *v)
{
    scope *p = v->myscope;
    int len = strlen(v->name);
    char *q;

    while (p != NULL) {
        len += strlen(p->me->name) + 1;
        p = p->parent;
    }
    len++;

    q = xmalloc(len);
    q[0] = '\0';
    appendName(v->myscope, q, len);
    strlcat(q, v->name, len);

    return q;
}

static void
freeValue(value *v)
{
    free(v->name);
    switch (v->type) {
    case scopeval:
        freeScopeTree(v->v.scope_val);
        break;

    case stringval:
        free(v->v.charp_val);
        break;

    default:
        break;
    }
    free(v);
}

static char *
checkName(scope *s, const char *name)
{
    int i;
    char *error = NULL;

    if (s == NULL)
        return NULL;

    for (i = 0; i < s->value_idx; i++) {
        char *n = NULL;

        if (strcmp(name, s->values[i]->name) == 0) {
            n = valueScopedName(s->values[i]);
            error = concat("Two definitions of ", n, (char *) 0);
            free(n);
            return error;
        }
    }

    return error;
}


static void
addValue(scope *s, value *v)
{
    v->myscope = s;

    if (s == NULL)
        return;

    if (s->value_count == s->value_idx) {
        if (s->values == 0) {
            s->values = (value **) xcalloc(10, sizeof(value *));
            s->value_count = 10;
        } else {
            s->value_count += 10;
            s->values = (value **) xrealloc(s->values,
                                            sizeof(value *) * s->value_count);
        }
    }

    s->values[s->value_idx++] = v;
}


static char *
addScope(scope *s, const char *name, scope *val)
{
    value *v;
    char *error;

    if ((error = checkName(s, name)) != NULL)
        return error;

    v = (value *) xcalloc(1, sizeof(value));
    v->name = xstrdup(name);
    v->type = scopeval;
    v->v.scope_val = val;
    val->me = v;
    val->parent = s;

    addValue(s, v);

    currScope = val;

    return NULL;
}


static void
printScope(FILE *fp, scope *s, int indent)
{
    int i;
    for (i = 0; i < s->value_idx; i++)
        printValue(fp, s->values[i], indent + 5);
}

static void
printValue(FILE *fp, value *v, int indent)
{
    int i;

    for (i = 0; i < indent; i++)
        fputc(' ', fp);

    switch (v->type) {
    case intval:
        fprintf(fp, "%s : %ld # INTEGER\n", v->name, v->v.int_val);
        break;

    case stringval:
        fprintf(fp, "%s : \"", v->name);
        {
            char *p = v->v.charp_val;
            while (*p) {
                if (*p == '"' || *p == '\\')
                    fputc('\\', fp);
                fputc(*p, fp);
                p++;
            }
        }
        fprintf(fp, "\" # STRING\n");
        break;

    case charval:
        fprintf(fp, "%s : %c", v->name, 047);
        switch (v->v.char_val) {
        case '\\':
            fprintf(fp, "\\\\");
            break;

        default:
            if (isprint((unsigned char) v->v.char_val))
                fprintf(fp, "%c", v->v.char_val);
            else
                fprintf(fp, "\\%03o", v->v.char_val);
        }
        fprintf(fp, "%c # CHARACTER\n", 047);
        break;

    case realval:
        fprintf(fp, "%s : %f # REAL\n", v->name, v->v.real_val);
        break;

    case boolval:
        fprintf(fp, "%s : %s # BOOLEAN\n", v->name,
                (v->v.bool_val ? "true" : "false"));
        break;

    case scopeval:
        fprintf(fp, "%s %s { # SCOPE\n", v->v.scope_val->scope_type, v->name);
        printScope(fp, v->v.scope_val, indent + 5);
        for (i = 0; i < indent; i++)
            fputc(' ', fp);
        fprintf(fp, "}\n");
        break;

    default:
        fprintf(fp, "UNKNOWN value type: %d\n", v->type);
        exit(1);
    }
}


static scope *
newScope(const char *type)
{
    scope *t;
    int i;

    t = (scope *) xcalloc(1, sizeof(scope));
    t->parent = NULL;
    t->scope_type = xstrdup(type);

    for (i = 0; t->scope_type[i] != '\0'; i++)
        t->scope_type[i] = tolower((unsigned char) t->scope_type[i]);

    return t;
}


#define BAD_KEY   "line %d: illegal key name: %s"
#define NON_ALPHA "line %d: keys must start with a letter: %s"

static char *
keyOk(const char *key)
{
    const char *p = key;
    char *rval;

    if (key == NULL) {
        rval = xmalloc(strlen("line : NULL key") + 15);
        sprintf(rval, "line %d: NULL key", lineCount);
        return rval;
    } else if (*key == '\0') {
        rval = xmalloc(strlen("line : EMPTY KEY") + 15);
        sprintf(rval, "line %d: EMPTY KEY", lineCount);
        return rval;
    }

    if (!isalpha((unsigned char) *p)) {
        rval = xmalloc(strlen(NON_ALPHA) + strlen(key) + 15);
        sprintf(rval, NON_ALPHA, lineCount, key);
        return rval;
    }

    p++;
    while (*p) {
        if (!(isalnum((unsigned char) *p) || *p == '_' || *p == '-')) {
            rval = xmalloc(strlen(BAD_KEY) + strlen(key) + 15);
            sprintf(rval, BAD_KEY, lineCount, key);
            return rval;
        }
        p++;
    }

    return NULL;
}

static PFIVP *funcs = NULL;
static void **args = NULL;
static int funcCount;
static int funcIdx;

void
configAddLoadCallback(PFIVP func, void *arg)
{
    if (func == NULL)
        return;

    if (funcIdx == funcCount) {
        funcCount += 10;
        if (funcs == NULL) {
            funcs = xmalloc(sizeof(PFIVP) * funcCount);
            args = xmalloc(sizeof(void *) * funcCount);
        } else {
            funcs = xrealloc(funcs, sizeof(PFIVP) * funcCount);
            args = xrealloc(args, sizeof(void *) * funcCount);
        }
    }

    args[funcIdx] = arg;
    funcs[funcIdx++] = func;
}


void
configRemoveLoadCallback(PFIVP func)
{
    int i, j;

    for (i = 0; i < funcIdx; i++)
        if (funcs[i] == func)
            break;

    for (j = i; j < funcIdx - 1; j++) {
        funcs[j] = funcs[j + 1];
        args[j] = args[j + 1];
    }

    if (funcIdx > 1 && i < funcIdx) {
        funcs[i - 2] = funcs[i - 1];
        args[i - 2] = args[i - 1];
    }

    if (funcIdx > 0 && i < funcIdx)
        funcIdx--;
}


static int
doCallbacks(void)
{
    int i;
    int rval = 1;

    for (i = 0; i < funcIdx; i++)
        if (funcs[i] != NULL)
            rval = (funcs[i](args[i]) && rval);

    return rval;
}


static char *key;
/* clang-format off */

#line 708 "y.tab.c" /* yacc.c:337  */
# ifndef YY_NULLPTR
#  if defined __cplusplus
#   if 201103L <= __cplusplus
#    define YY_NULLPTR nullptr
#   else
#    define YY_NULLPTR 0
#   endif
#  else
#   define YY_NULLPTR ((void*)0)
#  endif
# endif

/* Enabling verbose error messages.  */
#ifdef YYERROR_VERBOSE
# undef YYERROR_VERBOSE
# define YYERROR_VERBOSE 1
#else
# define YYERROR_VERBOSE 0
#endif

/* In a future release of Bison, this section will be replaced
   by #include "y.tab.h".  */
#ifndef YY_YY_Y_TAB_H_INCLUDED
# define YY_YY_Y_TAB_H_INCLUDED
/* Debug traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif
#if YYDEBUG
extern int yydebug;
#endif

/* Token type.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
  enum yytokentype
  {
    PEER = 258,
    GROUP = 259,
    IVAL = 260,
    RVAL = 261,
    NAME = 262,
    XSTRING = 263,
    SCOPE = 264,
    COLON = 265,
    LBRACE = 266,
    RBRACE = 267,
    TRUEBVAL = 268,
    FALSEBVAL = 269,
    CHAR = 270,
    WORD = 271,
    IP_ADDRESS = 272
  };
#endif
/* Tokens.  */
#define PEER 258
#define GROUP 259
#define IVAL 260
#define RVAL 261
#define NAME 262
#define XSTRING 263
#define SCOPE 264
#define COLON 265
#define LBRACE 266
#define RBRACE 267
#define TRUEBVAL 268
#define FALSEBVAL 269
#define CHAR 270
#define WORD 271
#define IP_ADDRESS 272

/* Value type.  */
#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED

union YYSTYPE
{
#line 639 "configfile.y" /* yacc.c:352  */

    scope *scp;
    value *val;
    char *name;
    int integer;
    double real;
    char *string;
    char chr;

#line 795 "y.tab.c" /* yacc.c:352  */
};

typedef union YYSTYPE YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define YYSTYPE_IS_DECLARED 1
#endif


extern YYSTYPE yylval;

int yyparse (void);

#endif /* !YY_YY_Y_TAB_H_INCLUDED  */



#ifdef short
# undef short
#endif

#ifdef YYTYPE_UINT8
typedef YYTYPE_UINT8 yytype_uint8;
#else
typedef unsigned char yytype_uint8;
#endif

#ifdef YYTYPE_INT8
typedef YYTYPE_INT8 yytype_int8;
#else
typedef signed char yytype_int8;
#endif

#ifdef YYTYPE_UINT16
typedef YYTYPE_UINT16 yytype_uint16;
#else
typedef unsigned short yytype_uint16;
#endif

#ifdef YYTYPE_INT16
typedef YYTYPE_INT16 yytype_int16;
#else
typedef short yytype_int16;
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif ! defined YYSIZE_T
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned
# endif
#endif

#define YYSIZE_MAXIMUM ((YYSIZE_T) -1)

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(Msgid) Msgid
# endif
#endif

#ifndef YY_ATTRIBUTE
# if (defined __GNUC__                                               \
      && (2 < __GNUC__ || (__GNUC__ == 2 && 96 <= __GNUC_MINOR__)))  \
     || defined __SUNPRO_C && 0x5110 <= __SUNPRO_C
#  define YY_ATTRIBUTE(Spec) __attribute__(Spec)
# else
#  define YY_ATTRIBUTE(Spec) /* empty */
# endif
#endif

#ifndef YY_ATTRIBUTE_PURE
# define YY_ATTRIBUTE_PURE   YY_ATTRIBUTE ((__pure__))
#endif

#ifndef YY_ATTRIBUTE_UNUSED
# define YY_ATTRIBUTE_UNUSED YY_ATTRIBUTE ((__unused__))
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YYUSE(E) ((void) (E))
#else
# define YYUSE(E) /* empty */
#endif

#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
/* Suppress an incorrect diagnostic about yylval being uninitialized.  */
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN \
    _Pragma ("GCC diagnostic push") \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")\
    _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
# define YY_IGNORE_MAYBE_UNINITIALIZED_END \
    _Pragma ("GCC diagnostic pop")
#else
# define YY_INITIAL_VALUE(Value) Value
#endif
#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_END
#endif
#ifndef YY_INITIAL_VALUE
# define YY_INITIAL_VALUE(Value) /* Nothing. */
#endif


#if ! defined yyoverflow || YYERROR_VERBOSE

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's 'empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
             && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* ! defined yyoverflow || YYERROR_VERBOSE */


#if (! defined yyoverflow \
     && (! defined __cplusplus \
         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yytype_int16 yyss_alloc;
  YYSTYPE yyvs_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)                           \
    do                                                                  \
      {                                                                 \
        YYSIZE_T yynewbytes;                                            \
        YYCOPY (&yyptr->Stack_alloc, Stack, yysize);                    \
        Stack = &yyptr->Stack_alloc;                                    \
        yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
        yyptr += yynewbytes / sizeof (*yyptr);                          \
      }                                                                 \
    while (0)

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from SRC to DST.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(Dst, Src, Count) \
      __builtin_memcpy (Dst, Src, (Count) * sizeof (*(Src)))
#  else
#   define YYCOPY(Dst, Src, Count)              \
      do                                        \
        {                                       \
          YYSIZE_T yyi;                         \
          for (yyi = 0; yyi < (Count); yyi++)   \
            (Dst)[yyi] = (Src)[yyi];            \
        }                                       \
      while (0)
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  3
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   30

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  18
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  10
/* YYNRULES -- Number of rules.  */
#define YYNRULES  21
/* YYNSTATES -- Number of states.  */
#define YYNSTATES  33

#define YYUNDEFTOK  2
#define YYMAXUTOK   272

/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex, with out-of-bounds checking.  */
#define YYTRANSLATE(YYX)                                                \
  ((unsigned) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)

/* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex.  */
static const yytype_uint8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17
};

#if YYDEBUG
  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
static const yytype_uint16 yyrline[] =
{
       0,   667,   667,   667,   676,   678,   679,   680,   686,   686,
     692,   692,   698,   705,   705,   712,   718,   723,   729,   735,
     740,   745
};
#endif

#if YYDEBUG || YYERROR_VERBOSE || 0
/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "$end", "error", "$undefined", "PEER", "GROUP", "IVAL", "RVAL", "NAME",
  "XSTRING", "SCOPE", "COLON", "LBRACE", "RBRACE", "TRUEBVAL", "FALSEBVAL",
  "CHAR", "WORD", "IP_ADDRESS", "$accept", "input", "$@1", "scope",
  "entries", "entry", "$@2", "$@3", "$@4", "value", YY_NULLPTR
};
#endif

# ifdef YYPRINT
/* YYTOKNUM[NUM] -- (External) token number corresponding to the
   (internal) symbol number NUM (which must be that of a token).  */
static const yytype_uint16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
     265,   266,   267,   268,   269,   270,   271,   272
};
# endif

#define YYPACT_NINF -12

#define yypact_value_is_default(Yystate) \
  (!!((Yystate) == (-12)))

#define YYTABLE_NINF -5

#define yytable_value_is_error(Yytable_value) \
  0

  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
     STATE-NUM.  */
static const yytype_int8 yypact[] =
{
     -12,     2,   -12,   -12,     0,   -12,   -11,    -9,    -6,   -12,
       1,     3,     4,     8,   -12,   -12,   -12,    14,   -12,   -12,
     -12,   -12,   -12,   -12,   -12,   -12,   -12,   -12,    -1,     5,
      11,   -12,   -12
};

  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
     Performed when YYTABLE does not specify something else to do.  Zero
     means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       2,     0,     5,     1,     0,     7,     0,     0,    13,     6,
       0,     0,     0,     0,     8,    10,    12,     0,     5,     5,
      16,    19,    20,    17,    18,    21,    15,    14,     0,     0,
       0,     9,    11
};

  /* YYPGOTO[NTERM-NUM].  */
static const yytype_int8 yypgoto[] =
{
     -12,   -12,   -12,     6,    22,   -12,   -12,   -12,   -12,   -12
};

  /* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int8 yydefgoto[] =
{
      -1,     1,     2,    28,    29,     9,    18,    19,    13,    27
};

  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
     positive, shift that token.  If negative, reduce the rule whose
     number is the opposite.  If YYTABLE_NINF, syntax error.  */
static const yytype_int8 yytable[] =
{
      -3,     5,     3,     6,     7,    10,     5,    11,     6,     7,
      12,    31,    14,     0,    15,    16,     8,    -4,    17,    20,
      21,     8,    22,    32,     4,    30,     0,    23,    24,    25,
      26
};

static const yytype_int8 yycheck[] =
{
       0,     1,     0,     3,     4,    16,     1,    16,     3,     4,
      16,    12,    11,    -1,    11,    11,    16,    12,    10,     5,
       6,    16,     8,    12,     2,    19,    -1,    13,    14,    15,
      16
};

  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
     symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,    19,    20,     0,    22,     1,     3,     4,    16,    23,
      16,    16,    16,    26,    11,    11,    11,    10,    24,    25,
       5,     6,     8,    13,    14,    15,    16,    27,    21,    22,
      21,    12,    12
};

  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    18,    20,    19,    21,    22,    22,    22,    24,    23,
      25,    23,    23,    26,    23,    27,    27,    27,    27,    27,
      27,    27
};

  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
static const yytype_uint8 yyr2[] =
{
       0,     2,     0,     2,     1,     0,     2,     2,     0,     6,
       0,     6,     3,     0,     4,     1,     1,     1,     1,     1,
       1,     1
};


#define yyerrok         (yyerrstatus = 0)
#define yyclearin       (yychar = YYEMPTY)
#define YYEMPTY         (-2)
#define YYEOF           0

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab


#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)                                    \
  do                                                              \
    if (yychar == YYEMPTY)                                        \
      {                                                           \
        yychar = (Token);                                         \
        yylval = (Value);                                         \
        YYPOPSTACK (yylen);                                       \
        yystate = *yyssp;                                         \
        goto yybackup;                                            \
      }                                                           \
    else                                                          \
      {                                                           \
        yyerror (YY_("syntax error: cannot back up")); \
        YYERROR;                                                  \
      }                                                           \
  while (0)

/* Error token number */
#define YYTERROR        1
#define YYERRCODE       256



/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)                        \
do {                                            \
  if (yydebug)                                  \
    YYFPRINTF Args;                             \
} while (0)

/* This macro is provided for backward compatibility. */
#ifndef YY_LOCATION_PRINT
# define YY_LOCATION_PRINT(File, Loc) ((void) 0)
#endif


# define YY_SYMBOL_PRINT(Title, Type, Value, Location)                    \
do {                                                                      \
  if (yydebug)                                                            \
    {                                                                     \
      YYFPRINTF (stderr, "%s ", Title);                                   \
      yy_symbol_print (stderr,                                            \
                  Type, Value); \
      YYFPRINTF (stderr, "\n");                                           \
    }                                                                     \
} while (0)


/*-----------------------------------.
| Print this symbol's value on YYO.  |
`-----------------------------------*/

static void
yy_symbol_value_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
{
  FILE *yyoutput = yyo;
  YYUSE (yyoutput);
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yytype < YYNTOKENS)
    YYPRINT (yyo, yytoknum[yytype], *yyvaluep);
# endif
  YYUSE (yytype);
}


/*---------------------------.
| Print this symbol on YYO.  |
`---------------------------*/

static void
yy_symbol_print (FILE *yyo, int yytype, YYSTYPE const * const yyvaluep)
{
  YYFPRINTF (yyo, "%s %s (",
             yytype < YYNTOKENS ? "token" : "nterm", yytname[yytype]);

  yy_symbol_value_print (yyo, yytype, yyvaluep);
  YYFPRINTF (yyo, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

static void
yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)                            \
do {                                                            \
  if (yydebug)                                                  \
    yy_stack_print ((Bottom), (Top));                           \
} while (0)


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

static void
yy_reduce_print (yytype_int16 *yyssp, YYSTYPE *yyvsp, int yyrule)
{
  unsigned long yylno = yyrline[yyrule];
  int yynrhs = yyr2[yyrule];
  int yyi;
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
             yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr,
                       yystos[yyssp[yyi + 1 - yynrhs]],
                       &yyvsp[(yyi + 1) - (yynrhs)]
                                              );
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)          \
do {                                    \
  if (yydebug)                          \
    yy_reduce_print (yyssp, yyvsp, Rule); \
} while (0)

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args)
# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif


#if YYERROR_VERBOSE

# ifndef yystrlen
#  if defined __GLIBC__ && defined _STRING_H
#   define yystrlen strlen
#  else
/* Return the length of YYSTR.  */
static YYSIZE_T
yystrlen (const char *yystr)
{
  YYSIZE_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
#  endif
# endif

# ifndef yystpcpy
#  if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#   define yystpcpy stpcpy
#  else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
static char *
yystpcpy (char *yydest, const char *yysrc)
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
#  endif
# endif

# ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYSIZE_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYSIZE_T yyn = 0;
      char const *yyp = yystr;

      for (;;)
        switch (*++yyp)
          {
          case '\'':
          case ',':
            goto do_not_strip_quotes;

          case '\\':
            if (*++yyp != '\\')
              goto do_not_strip_quotes;
            else
              goto append;

          append:
          default:
            if (yyres)
              yyres[yyn] = *yyp;
            yyn++;
            break;

          case '"':
            if (yyres)
              yyres[yyn] = '\0';
            return yyn;
          }
    do_not_strip_quotes: ;
    }

  if (! yyres)
    return yystrlen (yystr);

  return (YYSIZE_T) (yystpcpy (yyres, yystr) - yyres);
}
# endif

/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
   about the unexpected token YYTOKEN for the state stack whose top is
   YYSSP.

   Return 0 if *YYMSG was successfully written.  Return 1 if *YYMSG is
   not large enough to hold the message.  In that case, also set
   *YYMSG_ALLOC to the required number of bytes.  Return 2 if the
   required number of bytes is too large to store.  */
static int
yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                yytype_int16 *yyssp, int yytoken)
{
  YYSIZE_T yysize0 = yytnamerr (YY_NULLPTR, yytname[yytoken]);
  YYSIZE_T yysize = yysize0;
  enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
  /* Internationalized format string. */
  const char *yyformat = YY_NULLPTR;
  /* Arguments of yyformat. */
  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
  /* Number of reported tokens (one for the "unexpected", one per
     "expected"). */
  int yycount = 0;

  /* There are many possibilities here to consider:
     - If this state is a consistent state with a default action, then
       the only way this function was invoked is if the default action
       is an error action.  In that case, don't check for expected
       tokens because there are none.
     - The only way there can be no lookahead present (in yychar) is if
       this state is a consistent state with a default action.  Thus,
       detecting the absence of a lookahead is sufficient to determine
       that there is no unexpected or expected token to report.  In that
       case, just report a simple "syntax error".
     - Don't assume there isn't a lookahead just because this state is a
       consistent state with a default action.  There might have been a
       previous inconsistent state, consistent state with a non-default
       action, or user semantic action that manipulated yychar.
     - Of course, the expected token list depends on states to have
       correct lookahead information, and it depends on the parser not
       to perform extra reductions after fetching a lookahead from the
       scanner and before detecting a syntax error.  Thus, state merging
       (from LALR or IELR) and default reductions corrupt the expected
       token list.  However, the list is correct for canonical LR with
       one exception: it will still contain any token that will not be
       accepted due to an error action in a later state.
  */
  if (yytoken != YYEMPTY)
    {
      int yyn = yypact[*yyssp];
      yyarg[yycount++] = yytname[yytoken];
      if (!yypact_value_is_default (yyn))
        {
          /* Start YYX at -YYN if negative to avoid negative indexes in
             YYCHECK.  In other words, skip the first -YYN actions for
             this state because they are default actions.  */
          int yyxbegin = yyn < 0 ? -yyn : 0;
          /* Stay within bounds of both yycheck and yytname.  */
          int yychecklim = YYLAST - yyn + 1;
          int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
          int yyx;

          for (yyx = yyxbegin; yyx < yyxend; ++yyx)
            if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR
                && !yytable_value_is_error (yytable[yyx + yyn]))
              {
                if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
                  {
                    yycount = 1;
                    yysize = yysize0;
                    break;
                  }
                yyarg[yycount++] = yytname[yyx];
                {
                  YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULLPTR, yytname[yyx]);
                  if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
                    yysize = yysize1;
                  else
                    return 2;
                }
              }
        }
    }

  switch (yycount)
    {
# define YYCASE_(N, S)                      \
      case N:                               \
        yyformat = S;                       \
      break
    default: /* Avoid compiler warnings. */
      YYCASE_(0, YY_("syntax error"));
      YYCASE_(1, YY_("syntax error, unexpected %s"));
      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
# undef YYCASE_
    }

  {
    YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
    if (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM)
      yysize = yysize1;
    else
      return 2;
  }

  if (*yymsg_alloc < yysize)
    {
      *yymsg_alloc = 2 * yysize;
      if (! (yysize <= *yymsg_alloc
             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
      return 1;
    }

  /* Avoid sprintf, as that infringes on the user's name space.
     Don't have undefined behavior even if the translation
     produced a string with the wrong number of "%s"s.  */
  {
    char *yyp = *yymsg;
    int yyi = 0;
    while ((*yyp = *yyformat) != '\0')
      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
        {
          yyp += yytnamerr (yyp, yyarg[yyi++]);
          yyformat += 2;
        }
      else
        {
          yyp++;
          yyformat++;
        }
  }
  return 0;
}
#endif /* YYERROR_VERBOSE */

/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

static void
yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
{
  YYUSE (yyvaluep);
  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YYUSE (yytype);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}




/* The lookahead symbol.  */
int yychar;

/* The semantic value of the lookahead symbol.  */
YYSTYPE yylval;
/* Number of syntax errors so far.  */
int yynerrs;


/*----------.
| yyparse.  |
`----------*/

int
yyparse (void)
{
    int yystate;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus;

    /* The stacks and their tools:
       'yyss': related to states.
       'yyvs': related to semantic values.

       Refer to the stacks through separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* The state stack.  */
    yytype_int16 yyssa[YYINITDEPTH];
    yytype_int16 *yyss;
    yytype_int16 *yyssp;

    /* The semantic value stack.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs;
    YYSTYPE *yyvsp;

    YYSIZE_T yystacksize;

  int yyn;
  int yyresult;
  /* Lookahead token as an internal (translated) token number.  */
  int yytoken = 0;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;

#if YYERROR_VERBOSE
  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
#endif

#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  yyssp = yyss = yyssa;
  yyvsp = yyvs = yyvsa;
  yystacksize = YYINITDEPTH;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY; /* Cause a token to be read.  */
  goto yysetstate;


/*------------------------------------------------------------.
| yynewstate -- push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;


/*--------------------------------------------------------------------.
| yynewstate -- set current state (the top of the stack) to yystate.  |
`--------------------------------------------------------------------*/
yysetstate:
  *yyssp = (yytype_int16) yystate;

  if (yyss + yystacksize - 1 <= yyssp)
#if !defined yyoverflow && !defined YYSTACK_RELOCATE
    goto yyexhaustedlab;
#else
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYSIZE_T yysize = (YYSIZE_T) (yyssp - yyss + 1);

# if defined yyoverflow
      {
        /* Give user a chance to reallocate the stack.  Use copies of
           these so that the &'s don't force the real ones into
           memory.  */
        YYSTYPE *yyvs1 = yyvs;
        yytype_int16 *yyss1 = yyss;

        /* Each stack pointer address is followed by the size of the
           data in use in that stack, in bytes.  This used to be a
           conditional around just the two extra args, but that might
           be undefined if yyoverflow is a macro.  */
        yyoverflow (YY_("memory exhausted"),
                    &yyss1, yysize * sizeof (*yyssp),
                    &yyvs1, yysize * sizeof (*yyvsp),
                    &yystacksize);
        yyss = yyss1;
        yyvs = yyvs1;
      }
# else /* defined YYSTACK_RELOCATE */
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
        goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
        yystacksize = YYMAXDEPTH;

      {
        yytype_int16 *yyss1 = yyss;
        union yyalloc *yyptr =
          (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
        if (! yyptr)
          goto yyexhaustedlab;
        YYSTACK_RELOCATE (yyss_alloc, yyss);
        YYSTACK_RELOCATE (yyvs_alloc, yyvs);
# undef YYSTACK_RELOCATE
        if (yyss1 != yyssa)
          YYSTACK_FREE (yyss1);
      }
# endif

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;

      YYDPRINTF ((stderr, "Stack size increased to %lu\n",
                  (unsigned long) yystacksize));

      if (yyss + yystacksize - 1 <= yyssp)
        YYABORT;
    }
#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */

  YYDPRINTF ((stderr, "Entering state %d\n", yystate));

  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:
  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token: "));
      yychar = yylex ();
    }

  if (yychar <= YYEOF)
    {
      yychar = yytoken = YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);

  /* Discard the shifted token.  */
  yychar = YYEMPTY;

  yystate = yyn;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END

  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     '$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
        case 2:
#line 667 "configfile.y" /* yacc.c:1652  */
    {
        lineCount = 1;
        addScope(NULL, "", newScope(""));
        topScope = currScope;
    }
#line 1905 "y.tab.c" /* yacc.c:1652  */
    break;

  case 3:
#line 671 "configfile.y" /* yacc.c:1652  */
    {
        if (!doCallbacks())
            YYABORT;
    }
#line 1914 "y.tab.c" /* yacc.c:1652  */
    break;

  case 7:
#line 680 "configfile.y" /* yacc.c:1652  */
    {
        errbuff = xmalloc(strlen(SYNTAX_ERROR) + 12);
        sprintf(errbuff, SYNTAX_ERROR, lineCount);
        YYABORT;
    }
#line 1924 "y.tab.c" /* yacc.c:1652  */
    break;

  case 8:
#line 686 "configfile.y" /* yacc.c:1652  */
    {
        errbuff = addScope(currScope, (yyvsp[-1].name), newScope("peer"));
        free((yyvsp[-1].name));
        if (errbuff != NULL)
            YYABORT;
    }
#line 1935 "y.tab.c" /* yacc.c:1652  */
    break;

  case 9:
#line 691 "configfile.y" /* yacc.c:1652  */
    { currScope = currScope->parent; }
#line 1941 "y.tab.c" /* yacc.c:1652  */
    break;

  case 10:
#line 692 "configfile.y" /* yacc.c:1652  */
    {
        errbuff = addScope(currScope, (yyvsp[-1].name), newScope("group"));
        free((yyvsp[-1].name));
        if (errbuff != NULL)
            YYABORT;
    }
#line 1952 "y.tab.c" /* yacc.c:1652  */
    break;

  case 11:
#line 697 "configfile.y" /* yacc.c:1652  */
    { currScope = currScope->parent; }
#line 1958 "y.tab.c" /* yacc.c:1652  */
    break;

  case 12:
#line 698 "configfile.y" /* yacc.c:1652  */
    {
        errbuff = xmalloc(strlen(UNKNOWN_SCOPE_TYPE) + 15 + strlen((yyvsp[-2].name)));
        sprintf(errbuff, UNKNOWN_SCOPE_TYPE, lineCount, (yyvsp[-2].name));
        free((yyvsp[-2].name));
        free((yyvsp[-1].name));
        YYABORT;
    }
#line 1970 "y.tab.c" /* yacc.c:1652  */
    break;

  case 13:
#line 705 "configfile.y" /* yacc.c:1652  */
    {
    if ((errbuff = keyOk((yyvsp[0].name))) != NULL) {
        YYABORT;
    } else
        key = (yyvsp[0].name);
    }
#line 1981 "y.tab.c" /* yacc.c:1652  */
    break;

  case 15:
#line 712 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addString(currScope, key, (yyvsp[0].name))) != NULL)
            YYABORT;
        free(key);
        free((yyvsp[0].name));
    }
#line 1992 "y.tab.c" /* yacc.c:1652  */
    break;

  case 16:
#line 718 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addInteger(currScope, key, (yyvsp[0].integer))) != NULL)
            YYABORT;
        free(key);
    }
#line 2002 "y.tab.c" /* yacc.c:1652  */
    break;

  case 17:
#line 723 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addBoolean(currScope, key, 1)) != NULL)
            YYABORT;
        free(key);
        free((yyvsp[0].name));
    }
#line 2013 "y.tab.c" /* yacc.c:1652  */
    break;

  case 18:
#line 729 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addBoolean(currScope, key, 0)) != NULL)
            YYABORT;
        free(key);
        free((yyvsp[0].name));
    }
#line 2024 "y.tab.c" /* yacc.c:1652  */
    break;

  case 19:
#line 735 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addReal(currScope, key, (yyvsp[0].real))) != NULL)
            YYABORT;
        free(key);
    }
#line 2034 "y.tab.c" /* yacc.c:1652  */
    break;

  case 20:
#line 740 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addString(currScope, key, (yyvsp[0].string))) != NULL)
            YYABORT;
        free(key);
    }
#line 2044 "y.tab.c" /* yacc.c:1652  */
    break;

  case 21:
#line 745 "configfile.y" /* yacc.c:1652  */
    {
        if ((errbuff = addChar(currScope, key, (yyvsp[0].chr))) != NULL)
            YYABORT;
        free(key);
    }
#line 2054 "y.tab.c" /* yacc.c:1652  */
    break;


#line 2058 "y.tab.c" /* yacc.c:1652  */
      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);

  *++yyvsp = yyval;

  /* Now 'shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */
  {
    const int yylhs = yyr1[yyn] - YYNTOKENS;
    const int yyi = yypgoto[yylhs] + *yyssp;
    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
               ? yytable[yyi]
               : yydefgoto[yylhs]);
  }

  goto yynewstate;


/*--------------------------------------.
| yyerrlab -- here on detecting error.  |
`--------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == YYEMPTY ? YYEMPTY : YYTRANSLATE (yychar);

  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
#if ! YYERROR_VERBOSE
      yyerror (YY_("syntax error"));
#else
# define YYSYNTAX_ERROR yysyntax_error (&yymsg_alloc, &yymsg, \
                                        yyssp, yytoken)
      {
        char const *yymsgp = YY_("syntax error");
        int yysyntax_error_status;
        yysyntax_error_status = YYSYNTAX_ERROR;
        if (yysyntax_error_status == 0)
          yymsgp = yymsg;
        else if (yysyntax_error_status == 1)
          {
            if (yymsg != yymsgbuf)
              YYSTACK_FREE (yymsg);
            yymsg = (char *) YYSTACK_ALLOC (yymsg_alloc);
            if (!yymsg)
              {
                yymsg = yymsgbuf;
                yymsg_alloc = sizeof yymsgbuf;
                yysyntax_error_status = 2;
              }
            else
              {
                yysyntax_error_status = YYSYNTAX_ERROR;
                yymsgp = yymsg;
              }
          }
        yyerror (yymsgp);
        if (yysyntax_error_status == 2)
          goto yyexhaustedlab;
      }
# undef YYSYNTAX_ERROR
#endif
    }



  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
         error, discard it.  */

      if (yychar <= YYEOF)
        {
          /* Return failure if at end of input.  */
          if (yychar == YYEOF)
            YYABORT;
        }
      else
        {
          yydestruct ("Error: discarding",
                      yytoken, &yylval);
          yychar = YYEMPTY;
        }
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:
  /* Pacify compilers when the user code never invokes YYERROR and the
     label yyerrorlab therefore never appears in user code.  */
  if (0)
    YYERROR;

  /* Do not reclaim the symbols of the rule whose action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;      /* Each real token shifted decrements this.  */

  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
        {
          yyn += YYTERROR;
          if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
            {
              yyn = yytable[yyn];
              if (0 < yyn)
                break;
            }
        }

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
        YYABORT;


      yydestruct ("Error: popping",
                  yystos[yystate], yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;


/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;


#if !defined yyoverflow || YYERROR_VERBOSE
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  /* Fall through.  */
#endif


/*-----------------------------------------------------.
| yyreturn -- parsing is finished, return the result.  |
`-----------------------------------------------------*/
yyreturn:
  if (yychar != YYEMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval);
    }
  /* Do not reclaim the symbols of the rule whose action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
                  yystos[*yyssp], yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
#if YYERROR_VERBOSE
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
#endif
  return yyresult;
}
#line 751 "configfile.y" /* yacc.c:1918  */

/* clang-format on */

int
yyerror(const char *s)
{
#undef FMT
#define FMT "line %d: %s"

    errbuff = xmalloc(strlen(s) + strlen(FMT) + 20);
    sprintf(errbuff, FMT, lineCount, s);

    return 0;
}

int
yywrap(void)
{
    return 1;
}

extern FILE *yyin;
int yydebug;

#define NO_INHERIT 0


#if !defined(WANT_MAIN)

struct peer_table_s {
    char *peerName;
    value *peerValue;
};

static struct peer_table_s *peerTable;
static int peerTableCount;
static int peerTableIdx;

void
configCleanup(void)
{
    int i;

    for (i = 0; i < peerTableIdx; i++)
        free(peerTable[i].peerName);
    free(peerTable);

    freeScopeTree(topScope);
    free(funcs);
    free(args);
}


int
buildPeerTable(FILE *fp, scope *s)
{
    int rval = 1;
    int i, j;

    for (i = 0; i < s->value_idx; i++) {
        if (ISSCOPE(s->values[i]) && ISPEER(s->values[i])) {
            for (j = 0; j < peerTableIdx; j++) {
                if (strcmp(peerTable[j].peerName, s->values[i]->name) == 0) {
                    logOrPrint(LOG_ERR, fp,
                               "ME config: two peers with the same name: %s",
                               peerTable[j].peerName);
                    rval = 0;
                    break;
                }
            }

            if (j == peerTableIdx) {
                if (peerTableCount == peerTableIdx) {
                    peerTableCount += 10;
                    if (peerTable == NULL)
                        peerTable = xmalloc(sizeof(struct peer_table_s)
                                            * peerTableCount);
                    else
                        peerTable =
                            xrealloc(peerTable, sizeof(struct peer_table_s)
                                                    * peerTableCount);
                }

                peerTable[peerTableIdx].peerName = xstrdup(s->values[i]->name);
                peerTable[peerTableIdx].peerValue = s->values[i];
                peerTableIdx++;
            }
        } else if (ISSCOPE(s->values[i]))
            rval = (buildPeerTable(fp, s->values[i]->v.scope_val) && rval);
    }

    return rval;
}


/* read the config file. Any errors go to errorDest if it is non-NULL,
   otherwise they are syslogged. If justCheck is true then return after
   parsing */
static int inited = 0;
int
readConfig(const char *file, FILE *errorDest, int justCheck, int dump)
{
    scope *oldTop = topScope;
    FILE *fp;
    int rval;

    if (!inited) {
        inited = 1;
        yydebug = (getenv("YYDEBUG") == NULL ? 0 : 1);
        if (yydebug)
            atexit(configCleanup);
    }

    if (file == NULL || strlen(file) == 0 || !fileExistsP(file)) {
        logOrPrint(LOG_ERR, errorDest,
                   "ME config aborting, no such config file: %s",
                   file ? file : "(null)");
        d_printf(1, "No such config file: %s\n", file ? file : "(null)");
        exit(1);
    }

    if ((fp = fopen(file, "r")) == NULL) {
        logOrPrint(LOG_ERR, errorDest, "ME config aborting fopen %s: %s", file,
                   strerror(errno));
        exit(1);
    }

    logOrPrint(LOG_NOTICE, errorDest, "loading %s", file);

    yyin = fp;

    topScope = NULL;

    rval = yyparse();

    fclose(fp);

    if (rval != 0) /* failure */
    {
        freeScopeTree(topScope);
        if (justCheck)
            freeScopeTree(oldTop);
        else
            topScope = oldTop;
        topScope = NULL;

        if (errbuff != NULL) {
            if (errorDest != NULL)
                fprintf(errorDest, "config file error: %s\n", errbuff);
            else
                warn("ME config file error: %s", errbuff);

            free(errbuff);
        }

        return 0;
    }

    if (dump) {
        fprintf(errorDest ? errorDest : stderr, "Parsed config file:\n");
        printScope(errorDest ? errorDest : stderr, topScope, -5);
        fprintf(errorDest ? errorDest : stderr, "\n");
    }

    if (justCheck) {
        freeScopeTree(topScope);
        freeScopeTree(oldTop);

        topScope = NULL;
    } else {
        for (peerTableIdx--; peerTableIdx >= 0; peerTableIdx--) {
            free(peerTable[peerTableIdx].peerName);
            peerTable[peerTableIdx].peerName = NULL;
            peerTable[peerTableIdx].peerValue = NULL;
        }
        peerTableIdx = 0;

        if (!buildPeerTable(errorDest, topScope))
            logAndExit(1, "Failed to build list of peers");
    }

    return 1;
}


value *
getNextPeer(int *cookie)
{
    value *rval;

    if (*cookie < 0 || *cookie >= peerTableIdx)
        return NULL;

    rval = peerTable[*cookie].peerValue;

    (*cookie)++;

    return rval;
}


value *
findPeer(const char *name)
{
    value *v = NULL;
    int i;

    for (i = 0; i < peerTableIdx; i++)
        if (strcmp(peerTable[i].peerName, name) == 0) {
            v = peerTable[i].peerValue;
            break;
        }

    return v;
}

#endif

#if defined(WANT_MAIN)
int
main(int argc, char **argv)
{
    if (yyparse())
        printf("parsing failed: %s\n", errbuff ? errbuff : "NONE");
    else {
        printScope(stdout, topScope, -5);

        if (argc == 3) {
#    if defined(INNFEED_DEBUG)
          printf ("Looking for %s of type %s: ",argv[2],argv[1]) ;
          if (strncmp (argv[1],"int",3) == 0)
            {
              int i = 0 ;

              if (!getInteger (topScope,argv[2],&i))
                printf ("wasn't found.\n") ;
              else
                printf (" %d\n",i) ;
            }
          else if (strncmp (argv[1],"real",4) == 0)
            {
              double d = 0.0 ;

              if (!getReal (topScope,argv[2],&d))
                printf ("wasn't found.\n") ;
              else
                printf (" %0.5f\n",d) ;
            }
#    else
            value *v = findValue(topScope, argv[1], 1);

            if (v == NULL)
                printf("Can't find %s\n", argv[1]);
            else {
                long ival = 987654;

                if (getInteger(v->v.scope_val, argv[2], &ival, 1))
                    printf("Getting %s : %ld", argv[2], ival);
                else
                    printf("Name is not legal: %s\n", argv[2]);
            }
#    endif
        } else if (argc == 2) {
            value *v = findValue(topScope, argv[1], 1);

            if (v == NULL)
                printf("Can't find %s\n", argv[1]);
            else {
                printf("Getting %s : ", argv[1]);
                printValue(stdout, v, 0);
            }
        }
    }

    freeScopeTree(topScope);

    return 0;
}
#endif /* defined (WANT_MAIN) */
